#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "applibs_versions.h"
#include <applibs/log.h>
#include <applibs/rtc.h>

static time_t modifyTimeInSeconds = 10800;

static int rtc_get_time(struct timespec *time);
static int rtc_set_time(struct timespec *time);
static void rtc_print_time(struct timespec *time);

static int rtc_get_time(struct timespec *time)
{
    if (clock_gettime(CLOCK_REALTIME, time) == -1) {
        Log_Debug("ERROR: clock_gettime failed with error code: %s (%d).\n", strerror(errno),
                  errno);
        return -1;
    }

    return 0;
}

static int rtc_set_time(struct timespec *time)
{
    if (clock_settime(CLOCK_REALTIME, time) == -1) {
        Log_Debug("ERROR: clock_settime failed with error code: %s (%d).\n", strerror(errno),
                    errno);
        return -1;
    }

    if (clock_systohc() == -1) {
        Log_Debug("ERROR: clock_systohc failed with error code: %s (%d).\n", strerror(errno),
                    errno);
        return -1;
    }

    return 0;
}

static void rtc_print_time(struct timespec *time)
{
    char displayTimeBuffer[26];
    if (!asctime_r((gmtime(&time->tv_sec)), (char *restrict) & displayTimeBuffer)) {
        Log_Debug("ERROR: asctime_r failed with error code: %s (%d).\n", strerror(errno),
                    errno);
        return;
    }
    Log_Debug("UTC:%s", displayTimeBuffer);
}

int gdn_rtc_sample(void)
{ 
    struct timespec currentTime;
    if (0 != rtc_get_time(&currentTime)) {
        return -1;
    }
    rtc_print_time(&currentTime);

    currentTime.tv_sec += modifyTimeInSeconds;
    if (0 != rtc_set_time(&currentTime)) {
        return -1;
    }

    if (0 != rtc_get_time(&currentTime)) {
        return -1;
    }
    rtc_print_time(&currentTime);

    // resume rtc
    currentTime.tv_sec -= modifyTimeInSeconds;
    if (0 != rtc_set_time(&currentTime)) {
        return -1;
    }

    return 0;
}